package processing.android.dandelion;

import java.io.*;
import java.util.*;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;


//serial communication
public class Client implements Runnable {
	
	/* Bluetooth */
	private BluetoothAdapter mAdapter;
	private BluetoothDevice mDevice;
	private UUID uuid = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");

	/* Socket & streams for BT communication */
	private BluetoothSocket mSocket;
	private ConnectedThread mConnectedThread;
	private boolean got_adapter = false;
	private boolean connected = false;

	/* Buffer */
	private int bufferlength = 128;
	private int available = 0;
	private byte[] buffer;
	private byte[] rawbuffer;

	/* Debug variables */
	public static boolean DEBUG = false;
	public static String DEBUGTAG = "btserial 0.1.6 Debug message: ";
	Dandelion parent;

	public Client(Dandelion parent){
	//	
		this.parent=parent;
		new Handler(Looper.getMainLooper()).post(new Runnable() {
			@Override
			public void run() {
				mAdapter = BluetoothAdapter.getDefaultAdapter();
				got_adapter=true;
			}
		});
	}
	
	
	public String[] list() {
		Vector<String> list = new Vector<String>();
		Set<BluetoothDevice> devices;
		while (!got_adapter){
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
			}
		}
		try {
			devices = mAdapter.getBondedDevices();
			// convert the devices 'set' into an array so that we can
			// perform string functions on it
			Object[] deviceArray = devices.toArray();
			// step through it and assign each device in turn to
			// remoteDevice and then print it's name
			for (int i = 0; i < devices.size(); i++) {
				BluetoothDevice thisDevice = mAdapter
				.getRemoteDevice(deviceArray[i].toString());
				list.addElement(thisDevice.getAddress());
			}
		} catch (UnsatisfiedLinkError e) {
			// errorMessage("devices", e);
		} catch (Exception e) {
			// errorMessage("devices", e);
		}

		String outgoing[] = new String[list.size()];
		list.copyInto(outgoing);
		return outgoing;
	}
	
	
	public synchronized boolean connect(String mac) {
		while (!got_adapter){
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
			}
		}
		/* Before we connect, make sure to cancel any discovery! */
		if (mAdapter.isDiscovering()) {
			mAdapter.cancelDiscovery();
			if (DEBUG) Log.i("System.out", "Cancelled ongoing discovery");
		}

		/* Make sure we're using a real bluetooth address to connect with */
		if (BluetoothAdapter.checkBluetoothAddress(mac)) {
			/* Get the remote device we're trying to connect to */
			mDevice = mAdapter.getRemoteDevice(mac);
			/* Create the RFCOMM sockets */
			try {
				mSocket = mDevice.createRfcommSocketToServiceRecord(uuid);
				mSocket.connect();

				/* Set the status */
				connected = true;

				// Start the thread to manage the connection and perform transmissions
				mConnectedThread = new ConnectedThread(this,mSocket, bufferlength);
				mConnectedThread.start();

				if (DEBUG) Log.i("System.out", "Connected to device " + mDevice.getName()
						+ " [" + mDevice.getAddress() + "]");

				return true;
			} catch (IOException e) {
				connected = false;
				e.printStackTrace();
				return false;
			}

		} else {
			connected = false;
			if (DEBUG) Log.i("System.out", "Addres is not Bluetooth, please verify MAC.");

			return false;
		}
	}
	
	public synchronized void disconnect() {
		if (connected) {
			try {
				// kill the connected thread if it's running:
				if (mConnectedThread != null) {
					mConnectedThread.cancel();
					mConnectedThread = null;
				}

				/* Close the socket */
				mSocket.close();

				/* Set the connected state */
				connected = false;
				/* If it successfully closes I guess we just return a success? */
				//return 0;
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				/* Otherwise we'll go ahead and say "no, this didn't work well!" */
				//return 1;
			}
		}
	}
	
	
	/**
	 * Writes a byte[] buffer to the output stream.
	 * 
	 * @param buffer
	 */
	public  void write(byte[] buffer) {
		// Create temporary object
		ConnectedThread r;
		// Synchronize a copy of the ConnectedThread
		synchronized (this) {
			if (!connected) return;
			r = mConnectedThread;
		}
		// Perform the write unsynchronized
		r.write(buffer);
	}

	/**
	 * Writes a  String to the output stream.
	 * 
	 * @param thisString
	 */
	public  void write(String thisString) {
		byte[] thisBuffer = thisString.getBytes();
		write(thisBuffer);
	}

	/**
	 * Writes a  String to the output stream.
	 * 
	 * @param buffer
	 */
	public  void write(int thisInt) {
		byte[] thisBuffer = {(byte)thisInt};
		write(thisBuffer);
	}
	
	//Send command to dandelion and wait up to 1000ms for response
	public String sendDandelion(String code, boolean answer) throws TimeoutException,IOException{
		String returnCode = "";
		
		if(!answer){
			write(code);
			return returnCode;
		}
		
		
		write(code);
		returnCode = readDandelion(200);
		
		
		return returnCode;
	}
	
	
	//Reading receiver buffer
	public synchronized String readDandelion(long timeout) throws TimeoutException{
//		try{
			
			long start = System.currentTimeMillis();
			int n = mConnectedThread.available();
			StringBuffer returnBuffer= new StringBuffer();

			while(returnBuffer.indexOf("\n")==-1){

					long now = System.currentTimeMillis();
					long rest = timeout - (now-start);

					if(rest <= 0){
						System.out.println("timeout return_  " +  Thread.currentThread().getName() );
						throw new TimeoutException("TimeoutException!! " + (now - start));
					}

					try{
						Thread.sleep(10);
					}catch(InterruptedException e){}
					
				n = mConnectedThread.available();
				if (n>0){
					byte buf[];
					buf=mConnectedThread.read();
					returnBuffer.append(new String(buf));
				}
				
			}

			return returnBuffer.toString();
//		}catch(IOException e){
//			e.printStackTrace();
//		}
//
//		return "";
	}
	
	
	
	public int readByte() {
		return mConnectedThread.readByte();
	}
	
	/**
	 * Returns the whole byte buffer.
	 * 
	 * @return
	 */
	public byte[] readBytes() {
		return mConnectedThread.read();
	}
	
	/**
	 * Returns the available number of bytes in the buffer.
	 * 
	 * @return
	 */
	public int available() {
		return mConnectedThread.available();
	}
	
	@Override
	public void run() {
//		/* Init the buffer */
//		buffer = new byte[bufferlength];
//		rawbuffer = new byte[bufferlength];
//		
//		/* Set the connected state */
//		connected = true;
//
//		while (connected) {
//			/* Read the available bytes into the buffer */
//			rawbuffer= mConnectedThread.read();
//			available = mConnectedThread.available();
//			/* Clone the raw buffer */
//			buffer = rawbuffer.clone();
//		}
	}
	
	
//	public void cleanSerialPort(){
//	
//		closeSerialPort();
//		try{
//			output.close();
//		}catch(IOException e){}
//		output = null;
//	}
	
	
/*	//Send command to dandelion and wait up to 1000ms for response
	public String sendDandelion(String code, boolean answer) throws TimeoutException,IOException{
		String returnCode = "";
		
		if(!answer){
			write(code);
			return returnCode;
		}
		
		
		write(code);
		returnCode = readDandelion(200);
		
		
		return returnCode;
	}
	
	//Reading receiver buffer
	public synchronized String readDandelion(long timeout) throws TimeoutException{
		try{
			InputStream input = port.getInputStream();
			long start = System.currentTimeMillis();
			int n = input.available();
			StringBuffer returnBuffer= new StringBuffer();

			while(returnBuffer.indexOf("\n")==-1){

					long now = System.currentTimeMillis();
					long rest = timeout - (now-start);

					if(rest <= 0){
						System.out.println("timeout return_  " +  Thread.currentThread().getName() );
						throw new TimeoutException("TimeoutException!! " + (now - start));
					}

					try{
						Thread.sleep(10);
					}catch(InterruptedException e){}
					
				n = input.available();

				byte buf[] = new byte[n];
				input.read(buf);
				returnBuffer.append(new String(buf));
				
			}

			return returnBuffer.toString();
		}catch(IOException e){
			e.printStackTrace();
		}

		return "";
	}
	*/
	//Specify a name to open a Dandelion
//	public boolean openDandelion(String pname){
//		if(openSerialPort(pname)){
//
//			System.out.println("open serial port");
//			String returnCode="";
//			while(!returnCode.startsWith("Dandelion")){
//				try{
//				returnCode = sendDandelion("?  ",true);
//				}catch(TimeoutException e){
//				}catch(IOException e){
//				}
//			}
//			return true;
//		}
//		return false;
//	}
	
//	//Check the ports in order to find the Dandelion
//	public boolean findDandelion(){
//		//Properties prop = System.getProperties();
//		//String os = prop.getProperty("os.name");
//		Enumeration<?> portList = CommPortIdentifier.getPortIdentifiers();
//		if(!portList.hasMoreElements()){
//			System.out.println("no elements :there is no com port");
//		}
//		System.out.println("finding Dandelion...");
//		while(portList.hasMoreElements()){
//			CommPortIdentifier portId = (CommPortIdentifier)portList.nextElement();
//			String pname = portId.getName();
//			if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
//				if(pname.startsWith("/dev/cu")){
//					continue;
//				}
//				System.out.println("found port .. " + pname);
//				
//				//mac pname.startsWith("/dev/cu.usbserial-")
//				//win pname.startsWith("COM")
//				if(openSerialPort(pname)){
//
//					
//					try{
//						//Verify the response of [?\n]
//						sendDandelion("Q\n",false);//Always send Q
//						Thread.sleep(1000);
//						String version = sendDandelion("?\n",true);
//						System.out.println("Dandelion firm version : " + version);
//						if(version.startsWith("Dandelion")){
//							return true;
//						}
//
//						//Could not find ...
//						cleanSerialPort();		
//						
//					}catch(TimeoutException e){
//						cleanSerialPort();
//						continue;
//					}catch(IOException e){
//						cleanSerialPort();
//						continue;
//					}catch(InterruptedException e){
//						e.getStackTrace();
//					}
//
//				}
//			}
//		}
//		
//		return false;
//	}
	
//	private void write(String what) throws IOException{
//		write(what.getBytes());
//	}
//	
//	
//	private void write(byte bytes[]) throws IOException{
//		output.write(bytes);
//		output.flush();
//		
//	}
	

//	private boolean openSerialPort(String portName){
//		try{
//			CommPortIdentifier portId = CommPortIdentifier.getPortIdentifier(portName);
//			
//			port = (SerialPort)portId.open("DandelionSerialPort", 1000);
//	    port.setSerialPortParams(rate, databits, stopbits, parity);
//
//	    output = port.getOutputStream();
//	    
//		}catch(NoSuchPortException e){
//			//this port does not exist
//			System.out.println("naiyo!");
//			return false;
//		}catch(PortInUseException e){
//			//Port is in use
//			System.out.println("this comm port is used by " + e.currentOwner + " !");
//			return false;
//		}catch(UnsupportedCommOperationException e){
//			return false;
//		}catch(IOException e){
//			//IO stream error
//			System.out.println("io stream error!");
//			return false;
//		}
//
//		//System.out.println("hiraita!");
//		return true;
//	}
//	
//	private void closeSerialPort(){
//		if(port!=null) port.close();
//		
//		System.out.println("closing " + port.getName());
//	}
	

}
